
let dparse = ref false
let dinstr = ref false
let dinterm = ref false
let dtrace = ref false
let parse_only = ref false
let execute = ref false
let ddasm = ref false
let ext_parser = ref "parser.byte"

let options = Arg.align [
  "-parse", Arg.Set dparse, " Output the serialized AST.";
  "-pp", Arg.Set_string ext_parser, " Set the parser executable.";
  "-dinterm", Arg.Set dinterm, " Display the intermediate code on stdout.";
  "-dinstr", Arg.Set dinstr, " Display the generated bytecode on stdout.";
  "-x", Arg.Set execute, "Execute the generated bytecode";
  "-ddasm", Arg.Set ddasm, " Display the disassembled bytecode.";
  "-dtrace", Arg.Set dtrace, " Display the execution trace.";
]


let load_file filename =
  let ic = open_in filename in
  set_binary_mode_in ic true;
  let n = in_channel_length ic in
  let s = String.create n in
  really_input ic s 0 n;
  close_in ic; s

let parse_file filename =
  let parser lexer lexbuf = try
    Parser.program lexer lexbuf
  with
  | Parser.Error -> Error.error "Parsing" (Position.cpos lexbuf) "Unknown error.\n"
  in
  SyntacticAnalysis.process
    ~lexer_init: (fun filename -> Lexing.from_channel (open_in filename))
    ~lexer_fun: Lexer.main
    ~parser_fun: parser
    ~input: filename

let parse_file_externally filename =
  let oc = Unix.open_process_in ("ocamlrun " ^ !ext_parser ^ " -parse " ^ filename) in
  try Marshal.from_channel oc
  with End_of_file -> failwith "parsing failed"

let filenames = ref []

let usage = "usage: clap [options] input1.{clap|clvm} ... inputN.{clap|clvm}"

let _ = Arg.parse options (fun s -> filenames := s :: !filenames) usage

let process_file filename =
  if Filename.check_suffix filename "clap" then begin
    if !dparse then begin Marshal.to_channel stdout (parse_file filename) []; exit 0 end;
    let ast = parse_file_externally filename in
    let i = Compil.program ast in
    if !dinterm then Printer.print (Printer.interm i);
    let c = Compil.interm i in
    if !dinstr then Printer.print (Printer.bytecode c);
    if !execute
    then begin
      let c' = Asm.disassemble (Asm.assemble c) in
      if !ddasm then Printer.print (Printer.bytecode c');
      Machine.exec ~trace:!dtrace (Machine.init c')
    end else
      let filename = Filename.chop_extension filename ^ ".clvm" in
      let oc = open_out filename in
      output_string oc (Asm.assemble c)
  end else if Filename.check_suffix filename "clvm" then begin
    let c = Asm.disassemble (load_file filename) in
    if !ddasm then Printer.print (Printer.bytecode c);
    Machine.exec ~trace:!dtrace (Machine.init c )
  end else failwith ("don't know what to do with file "^filename)

let () =
  if !filenames = [] then Arg.usage options usage
  else List.iter process_file !filenames
